#include "WolframAlpha.hpp"

#include "Utility.hpp"
#include <cpr/cpr.h>
#include <ctre.hpp>
#include <nlohmann/json.hpp>
#include <string>

WolframAlpha::WolframAlpha(const std::string& query, double amount) : m_json(fetchJson(query)), m_amount(amount) {}

double WolframAlpha::getCalories() {
  if (auto match = ctre::search<R"((?<=total calories\s)\d+)">(getPod())) {
    return std::stod(match.data()) * m_amount;
  }
  throw std::runtime_error("Calorie information not found.");
}

double WolframAlpha::getProtein() {
  auto        m = ctre::search<"(?<=protein\\s)(\\d+)\\s*(\\w+)">(getPod());
  double      d = m.get<1>().to_number();
  std::string u = m.get<2>().to_string();
  return u == "g" ? d : d / 1000;
}

double WolframAlpha::getCarbs() {
  auto        m = ctre::search<"(?<=total carbohydrates\\s)(\\d+)\\s*(\\w+)">(getPod());
  double      d = m.get<1>().to_number();
  std::string u = m.get<2>().to_string();
  return u == "g" ? d : d / 1000;
}

double WolframAlpha::getFat() {
  auto        m = ctre::search<"(?<=total fat\\s)(\\d+)\\s*(\\w+)">(getPod());
  double      d = m.get<1>().to_number();
  std::string u = m.get<2>().to_string();
  return u == "g" ? d : d / 1000;
}

nlohmann::json WolframAlpha::fetchJson(const std::string& query) {
  const auto    WOLFRAM_KEY = utl::getEnv("NUTRI_WA_KEY");
  cpr::Response json        = Get(cpr::Url{"https://api.wolframalpha.com/v2/query"},
                                  cpr::Parameters{{"input", query + "100g nutrition"}, {"appid", WOLFRAM_KEY}, {"output", "JSON"}, {"format", "plaintext"}});
  return nlohmann::json::parse(json.text);
}

std::string WolframAlpha::getPod() {
  return to_string(m_json["queryresult"]["pods"][1]["subpods"][0]["plaintext"]);
}

nlohmann::json m_json{};
double         m_amount{};
